% This script samples the log-volatility
%
% See:
% Grant, A.L. and Chan, J.C.C. (2017). Measuring the Output Gap Using 
% Stochastic Model Specification Search, CAMA Working Paper 2/2017.

function [h,S] = SVRW(Ystar,h,sig,h0,Vh)

T = length(h);
%% normal mixture
pi = [0.0073 .10556 .00002 .04395 .34001 .24566 .2575];
mi = [-10.12999 -3.97281 -8.56686 2.77786 .61942 1.79518 -1.08819] - 1.2704;  %% means already adjusted!! %%
sigi = [5.79596 2.61369 5.17950 .16735 .64009 .34023 1.26261];
sqrtsigi = sqrt(sigi);

%% sample S from a 7-point distrete distribution
temprand = rand(T,1);
q = repmat(pi,T,1).*normpdf(repmat(Ystar,1,7),repmat(h,1,7)+repmat(mi,T,1), repmat(sqrtsigi,T,1));
q = q./repmat(sum(q,2),1,7);
S = 7 - sum(repmat(temprand,1,7)<cumsum(q,2),2)+1;
    
%% sample h
H = speye(T) - sparse(2:T,1:(T-1),ones(1,T-1),T,T);
alp = H\[h0; sparse(T-1,1)];
invSh = sparse(1:T,1:T,[1/Vh; 1/sig*ones(T-1,1)]);
muS = mi(S)'; invSigS = sparse(1:T,1:T,1./sigi(S));
Kh = H'*invSh*H + invSigS;
hhat = Kh\(H'*invSh*H*alp+invSigS*(Ystar-muS));
h = hhat + chol(Kh,'lower')'\randn(T,1);
end
