% This script evaluates the joint distribution of del_lam_u, del_gam_u, del_lam_y
%
% See:
% Grant, A.L. and Chan, J.C.C. (2017). Measuring the Output Gap Using 
% Stochastic Model Specification Search, CAMA Working Paper 2/2017.

function lden = lpost_del_var(del,pi,y,u,pistar,ystar,ustar,...
    Hrhoc,iSc,iSpi,prior)
lamu0 = prior(1);
gamu0 = prior(2);
lamy0 = prior(3);
rhopi0 = prior(4);
Vlamu = prior(5);
Vgamu = prior(6);
Vlamy = prior(7);
Vrhopi = prior(8);
p = prior(9:end);
del_lamu = del(1);
del_gamu = del(2);
del_lamy = del(3);

HrhociScHrhoc = Hrhoc'*iSc*Hrhoc;
Xpi = [[0;pi(1:end-1)-pistar(1:end-1)] del_lamu*(u-ustar) del_lamy*(y-ystar)];
K_betapi = diag([1/Vrhopi 1/Vlamu 1/Vlamy]) + Xpi'*iSpi*Xpi;
betapi_hat = K_betapi\([rhopi0/Vrhopi; lamu0/Vlamu; lamy0/Vlamy] ...
    + Xpi'*iSpi*(pi-pistar));
Kgamu = 1/Vgamu + del_gamu^2*(u-ustar)'*HrhociScHrhoc*(u-ustar);
gamuhat = Kgamu\(gamu0/Vgamu + del_gamu*(u-ustar)'*HrhociScHrhoc*(y-ystar));
lden = -sum(log(diag(chol(K_betapi)))) + .5*betapi_hat'*K_betapi*betapi_hat ...
    -.5*log(Kgamu) + .5*gamuhat^2*Kgamu + sum(del.*log(p)+(1-del).*log(1-p));
end